@@ -1,6 +1,14 @@
 #!/bin/sh
 echo "*** copying swig files"
 cp -a ../*.i .
+cp -a ../*.h .
+
+echo "*** Recreating libtool"
+if test -z "$LTIZE"; then
+LTIZE="$AUTODIR""libtoolize"
+fi
+echo "$LTIZE"
+       $LTIZE --force --copy;
 
 ACLOCAL="$AUTODIR""aclocal"
 echo "*** Recreating aclocal.m4"
