@@ -2,7 +2,7 @@
  *  swlocale.h   - definition of Class SWLocale used for retrieval
  *				of locale lookups
  *
- * $Id: swlocale.h 1864 2005-11-20 06:06:40Z scribe $
+ * $Id: swlocale.h 2191 2008-08-16 16:16:03Z scribe $
  *
  * Copyright 2000 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -48,8 +48,9 @@ class Private;
 	char *description;
 	char *encoding;
 	struct abbrev *bookAbbrevs;
-	char *BMAX;
-	struct sbook **books;
+	int abbrevsCnt;
+	const char **bookLongNames;
+	const char **bookPrefAbbrev;
 
 public:
 	SWLocale(const char *ifilename);
@@ -68,8 +69,8 @@ public:
 	virtual const char *translate(const char *text);
 	virtual void augment(SWLocale &addFrom);
 	virtual SWLocale & operator +=(SWLocale &addFrom) { augment(addFrom); return *this; }
-	virtual const struct abbrev *getBookAbbrevs();
-	virtual void getBooks(char **iBMAX, struct sbook ***ibooks);
+	virtual const struct abbrev *getBookAbbrevs(int *retSize);
+	static const char *DEFAULT_LOCALE_NAME;
 };
 
 SWORD_NAMESPACE_END
