@@ -3,7 +3,7 @@
  *		  types of modules (e.g. texts, commentaries, maps, lexicons,
  *		  etc.)
  *
- * $Id: swmodule.h 2093 2007-09-28 19:20:22Z scribe $
+ * $Id: swmodule.h 2318 2009-04-10 21:22:16Z scribe $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -78,7 +78,7 @@ typedef std::map < SWBuf, AttributeList, std::less < SWBuf > > AttributeTypeList
 // cache data.  But if we don't do this, then we need another mechanism to
 // check if we are an SWCacher.  Maybe make SWModule extend SWObject (which
 // it probably should anyway.  But then we need to add all the cheezy
-// heirarchy info to all he decendent classes for our SWDYNAMIC_CAST and
+// heirarchy info to all the decendent classes for our SWDYNAMIC_CAST and
 // then we can see if we implement SWCacher so we know whether or not to add
 // to the yet to be developed cachemgr.
 // Just leave for now.  This lets us always able to call module->flush()
@@ -103,7 +103,7 @@ protected:
 	char *modname;
 	char *moddesc;
 	char *modtype;
-	char *modlang;  
+	char *modlang;
 
 	char direction;
 	char markup;
@@ -133,6 +133,8 @@ protected:
 	int entrySize;
 	mutable long entryIndex;	 // internal common storage for index
 
+	static void prepText(SWBuf &buf);
+
 
 public:
 
@@ -153,7 +155,7 @@ public:
 	 * @param markup Source Markup of the module (e.g. OSIS)
 	 * @param modlang Language of the module (e.g. en)
 	 */
-	SWModule(const char *imodname = 0, const char *imoddesc = 0, SWDisplay * idisp = 0, char *imodtype = 0, SWTextEncoding encoding = ENC_UNKNOWN, SWTextDirection dir = DIRECTION_LTR, SWTextMarkup markup = FMT_UNKNOWN, const char* modlang = 0);
+	SWModule(const char *imodname = 0, const char *imoddesc = 0, SWDisplay * idisp = 0, const char *imodtype = 0, SWTextEncoding encoding = ENC_UNKNOWN, SWTextDirection dir = DIRECTION_LTR, SWTextMarkup markup = FMT_UNKNOWN, const char *modlang = 0);
 
 	/** SWModule d-tor
 	 */
@@ -168,7 +170,7 @@ public:
 	/**
 	 * @return  True if this module is encoded in Unicode, otherwise returns false.
 	 */
-	virtual const bool isUnicode() const { return (encoding == (char)ENC_UTF8 || encoding == (char)ENC_SCSU); }
+	virtual bool isUnicode() const { return (encoding == (char)ENC_UTF8); }
 
 	// These methods are useful for modules that come from a standard SWORD install (most do).
 	// SWMgr will call setConfig.  The user may use getConfig and getConfigEntry (if they
@@ -180,7 +182,7 @@ public:
 	/**
 	 * @return The size of the text entry for the module's current key position.
 	 */
-	virtual const int getEntrySize() const { return entrySize; }
+	virtual int getEntrySize() const { return entrySize; }
 
 	/**
 	 * Sets a key to this module for position to a particular record
@@ -204,7 +206,7 @@ public:
 	 * @deprecated Use setKey() instead.
 	 */
 	char SetKey(const SWKey &ikey) { return setKey(ikey); }
-	/** 
+	/**
 	 * @deprecated Use setKey() instead.
 	 */
 	char Key(const SWKey & ikey) { return setKey(ikey); }
@@ -233,12 +235,12 @@ public:
 	 * gets the key text for the module.
 	 * do we really need this?
 	 */
-	 
+
 	virtual const char *getKeyText() const {
 		return *getKey();
 	}
 
-	
+
 	virtual long Index() const { return entryIndex; }
 	virtual long Index(long iindex) { entryIndex = iindex; return entryIndex; }
 
@@ -260,7 +262,7 @@ public:
 	 */
 	virtual void setDisplay(SWDisplay * idisp);
 
-	/** 
+	/**
 	 * @deprecated Use get/setDisplay() instead.
 	 */
 	SWDisplay *Disp(SWDisplay * idisp = 0) {
@@ -379,7 +381,7 @@ public:
 	 * @see VerseKey, ListKey, SWText, SWLD, SWCom
 	 * @return pointer to allocated key
 	 */
-	virtual SWKey *CreateKey();
+	virtual SWKey *CreateKey() const;
 
 	/** This function is reimplemented by the different kinds
 	 * of module objects
@@ -436,19 +438,19 @@ public:
 	 * @param buf the buffer to filter
 	 * @param key key location from where this buffer was extracted
 	 */
-	virtual void filterBuffer(OptionFilterList *filters, SWBuf &buf, SWKey *key);
+	virtual void filterBuffer(OptionFilterList *filters, SWBuf &buf, const SWKey *key);
 
 	/** FilterBuffer a text buffer
 	 * @param filters the FilterList of filters to iterate
 	 * @param buf the buffer to filter
 	 * @param key key location from where this buffer was extracted
 	 */
-	virtual void filterBuffer(FilterList *filters, SWBuf &buf, SWKey *key);
+	virtual void filterBuffer(FilterList *filters, SWBuf &buf, const SWKey *key);
 
 	/** Adds a RenderFilter to this module's renderFilters queue.
 	 *	Render Filters are called when the module is asked to produce
 	 *	renderable text.
-	 * @param newFilter the filter to add
+	 * @param newfilter the filter to add
 	 * @return *this
 	 */
 	virtual SWModule &AddRenderFilter(SWFilter *newfilter) {
@@ -491,7 +493,7 @@ public:
 	 * @param buf the buffer to filter
 	 * @param key key location from where this buffer was extracted
 	 */
-	virtual void renderFilter(SWBuf &buf, SWKey *key) {
+	virtual void renderFilter(SWBuf &buf, const SWKey *key) {
 		filterBuffer(renderFilters, buf, key);
 	}
 
@@ -534,7 +536,7 @@ public:
 	 * @param buf the buffer to filter
 	 * @param key key location from where this buffer was extracted
 	 */
-	virtual void encodingFilter(SWBuf &buf, SWKey *key) {
+	virtual void encodingFilter(SWBuf &buf, const SWKey *key) {
 		filterBuffer(encodingFilters, buf, key);
 	}
 
@@ -562,7 +564,7 @@ public:
 	 * @param buf the buffer to filter
 	 * @param key key location from where this buffer was extracted
 	 */
-	virtual void stripFilter(SWBuf &buf, SWKey *key) {
+	virtual void stripFilter(SWBuf &buf, const SWKey *key) {
 		filterBuffer(stripFilters, buf, key);
 	}
 
@@ -571,7 +573,7 @@ public:
 	 * @param buf the buffer to filter
 	 * @param key key location from where this buffer was extracted
 	 */
-	virtual void rawFilter(SWBuf &buf, SWKey *key) {
+	virtual void rawFilter(SWBuf &buf, const SWKey *key) {
 		filterBuffer(rawFilters, buf, key);
 	}
 
@@ -591,7 +593,7 @@ public:
 	 * @param buf the buffer to filter
 	 * @param key key location from where this buffer was extracted
 	 */
-	virtual void optionFilter(SWBuf &buf, SWKey *key) {
+	virtual void optionFilter(SWBuf &buf, const SWKey *key) {
 		filterBuffer(optionFilters, buf, key);
 	}
 
@@ -619,13 +621,13 @@ public:
 	 * @param tmpKey desired module entry
 	 * @return result buffer
 	 */
-	virtual const char *StripText(SWKey *tmpKey);
+	virtual const char *StripText(const SWKey *tmpKey);
 
 	/** Produces renderable text of the module entry at the supplied key
 	 * @param tmpKey key to use to grab text
 	 * @return this module's text at specified key location massaged by Render filters
 	 */
-	virtual const char *RenderText(SWKey *tmpKey);
+	virtual const char *RenderText(const SWKey *tmpKey);
 
 	/** Whether or not to only hit one entry when iterating encounters
 	 *	consecutive links when iterating
@@ -637,6 +639,9 @@ public:
 	 *	consecutive links when iterating
 	 */
 	virtual bool getSkipConsecutiveLinks() { return skipConsecutiveLinks; }
+	
+	virtual bool isLinked(const SWKey *k1, const SWKey *k2) const { return false; }
+	virtual bool hasEntry(const SWKey *k) const { return false; }
 
 	/** Entry Attributes are special data pertaining to the current entry.
 	 *	To see what Entry Attributes exists for a specific entry of a module,
@@ -657,14 +662,15 @@ public:
 	 */
 	virtual bool isProcessEntryAttributes() const { return procEntAttr; }
 
-	// OPERATORS -----------------------------------------------------------------
-	
+
+	// SWSearchable Interface Impl -----------------------------------------------
 	virtual signed char createSearchFramework(
 			void (*percent) (char, void *) = &nullPercent,
 			void *percentUserData = 0);
 	virtual void deleteSearchFramework();
 	virtual bool hasSearchFramework();
 
+	// OPERATORS -----------------------------------------------------------------
 	SWMODULE_OPERATORS
 
 };
