@@ -2,16 +2,35 @@
  *  swkey.cpp - code for base class 'SWKey'.  SWKey is the basis for all
  *	        types of keys for indexing into modules (e.g. verse, word,
  *		place, etc.)
+ *
+ *
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
  */
 
+
 #include <swkey.h>
 #include <utilstr.h>
 #include <string.h>
+#include <localemgr.h>
 
 SWORD_NAMESPACE_START
 
 static const char *classes[] = {"SWKey", "SWObject", 0};
 SWClass SWKey::classdef(classes);
+SWKey::LocaleCache   SWKey::localeCache;
 
 /******************************************************************************
  * SWKey Constructor - initializes instance of SWKey
@@ -21,6 +40,7 @@ SWClass SWKey::classdef(classes);
 
 SWKey::SWKey(const char *ikey)
 {
+	init();
 	index     = 0;
 	persist   = 0;
 	keytext   = 0;
@@ -28,11 +48,12 @@ SWKey::SWKey(const char *ikey)
 	error     = 0;
 	userData  = 0;
 	stdstr(&keytext, ikey);
-	init();
 }
 
 SWKey::SWKey(SWKey const &k)
 {
+	init();
+	stdstr(&localeName, k.localeName);
 	index     = k.index;
 	persist   = k.persist;
 	userData  = k.userData;
@@ -40,12 +61,14 @@ SWKey::SWKey(SWKey const &k)
 	rangeText = 0;
 	error     = k.error;
 	setText(k.getText());
-	init();
 }
 
 void SWKey::init() {
 	myclass = &classdef;
 	boundSet = false;
+	locale = 0;
+	localeName = 0;
+	setLocale(LocaleMgr::getSystemLocaleMgr()->getDefaultLocaleName());
 }
 
 SWKey *SWKey::clone() const
@@ -58,10 +81,9 @@ SWKey *SWKey::clone() const
  */
 
 SWKey::~SWKey() {
-	if (keytext)
-		delete [] keytext;
-	if (rangeText)
-		delete [] rangeText;
+	delete [] keytext;
+	delete [] rangeText;
+	delete [] localeName;
 }
 
 
@@ -79,6 +101,28 @@ char SWKey::Persist() const
 }
 
 
+/******************************************************************************
+ * SWKey::getPrivateLocale - Gets a real locale object from our name
+ *
+ * RET:	locale object associated with our name
+ */
+
+SWLocale *SWKey::getPrivateLocale() const {
+	if (!locale) {
+		if ((!localeCache.name) || (strcmp(localeCache.name, localeName))) {
+			stdstr(&(localeCache.name), localeName);
+			// this line is the entire bit of work we're trying to avoid with the cache
+			// worth it?  compare time examples/cmdline/search KJV "God love world" to
+			// same with this method reduced to:
+			// if (!local) local = ... (call below); return locale;
+			localeCache.locale = LocaleMgr::getSystemLocaleMgr()->getLocale(localeName);
+		}
+		locale = localeCache.locale;
+	}
+	return locale;
+}
+
+
 /******************************************************************************
  * SWKey::Persist - Set/gets whether this object itself persists within a
  *			module that it was used to setKey or just a copy.
@@ -133,6 +177,7 @@ void SWKey::setText(const char *ikey) {
 
 void SWKey::copyFrom(const SWKey &ikey) {
 // not desirable	Persist(ikey.Persist());
+	setLocale(ikey.getLocale());
 	setText((const char *)ikey);
 }
 
@@ -156,6 +201,15 @@ const char *SWKey::getRangeText() const {
 }
 
 
+/******************************************************************************
+ * SWKey::getOSISRefRangeText - returns parsable range text for this key
+ */
+
+const char *SWKey::getOSISRefRangeText() const {
+	return getRangeText();
+}
+
+
 /******************************************************************************
  * SWKey::compare	- Compares another VerseKey object
  *
