@@ -491,6 +491,7 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 	char lastPartial = 0;
 	bool inTerm = true;
 	int notAllDigits = 0;
+	bool doubleF = false;
 
 	// assert we have a buffer
 	if (!buf) return internalListKey;
@@ -643,6 +644,9 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 //				if (comma) {
 					curKey->Chapter(lastKey->Chapter());
 					curKey->Verse(chap);  // chap because this is the first number captured
+					if (suffix) {
+						curKey->setSuffix(suffix);
+					}
 				}
 				else {
 					if (useChapterAsVerse && verse < 0 && chap > 0 && curKey->getChapterMax() == 1) {
@@ -693,7 +697,14 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 							tmpListKey.GetElement()->userData = (void *)(bufStart+(buf-iBuf.c_str()));
 						}
 						else {
+							bool f = false;
+							if (curKey->getSuffix() == 'f') {
+								curKey->setSuffix(0);
+								f = true;
+							}
 							lastKey->LowerBound(*curKey);
+							if (f && doubleF) (*curKey) = MAXVERSE;
+							else if (f) (*curKey)++;
 							lastKey->UpperBound(*curKey);
 							*lastKey = TOP;
 							tmpListKey << *lastKey;
@@ -762,7 +773,9 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 					break;
 				default:
 					// suffixes (and oddly 'f'-- ff.)
-					if ((*buf >= 'a' && *buf <= 'z') && (chap >=0)) {
+					if (((*buf >= 'a' && *buf <= 'z') && (chap >=0)) || *buf == 'f') {
+						// if suffix is already an 'f', then we need to mark if we're doubleF.
+						doubleF = (*buf == 'f' && suffix == 'f');
 						suffix = *buf;
 					}
 					else {
@@ -854,6 +867,9 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 		if (((comma)||((verse < 0)&&(bookno < 0)))&&(!lastPartial)) {
 			curKey->Chapter(lastKey->Chapter());
 			curKey->Verse(chap);  // chap because this is the first number captured
+			if (suffix) {
+				curKey->setSuffix(suffix);
+			}
 		}
 		else {
 			if (useChapterAsVerse && verse < 0 && chap > 0 && curKey->getChapterMax() == 1) {
@@ -901,11 +917,17 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 					tmpListKey.GetElement()->userData = (void *)(bufStart+(buf-iBuf.c_str()));
 				}
 				else {
+					bool f = false;
+					if (curKey->getSuffix() == 'f') {
+						curKey->setSuffix(0);
+						f = true;
+					}
 					lastKey->LowerBound(*curKey);
+					if (f && doubleF) (*curKey) = MAXVERSE;
+					else if (f) (*curKey)++;
 					lastKey->UpperBound(*curKey);
 					*lastKey = TOP;
 					tmpListKey << *lastKey;
-//					tmpListKey << curKey->getText();
 					tmpListKey.GetElement()->userData = (void *)(bufStart+(buf-iBuf.c_str()));
 				}
 			}
