@@ -2,7 +2,7 @@
  *  swlocale.cpp   - implementation of Class SWLocale used for retrieval
  *				of locale lookups
  *
- * $Id: swlocale.cpp 2346 2009-04-27 01:53:58Z scribe $
+ * $Id: swlocale.cpp 2463 2009-10-14 22:14:55Z chrislit $
  *
  * Copyright 2000 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -31,7 +31,7 @@ SWORD_NAMESPACE_START
 
 typedef std::map < SWBuf, SWBuf, std::less < SWBuf > >LookupMap;
 
-const char *SWLocale::DEFAULT_LOCALE_NAME="en_US";
+const char *SWLocale::DEFAULT_LOCALE_NAME="en";
 
 // I have bridge patterns, but this hides swconfig and map from lots o stuff
 class SWLocale::Private {
