@@ -83,8 +83,8 @@ OSISHTMLHREF::OSISHTMLHREF() {
 
 	setTokenCaseSensitive(true);
 	
-	addTokenSubstitute("lg",  "<br />");
-	addTokenSubstitute("/lg", "<br />");
+	//	addTokenSubstitute("lg",  "<br />");
+	//	addTokenSubstitute("/lg", "<br />");
 
 	morphFirst = false;
 }
@@ -194,11 +194,17 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 					outText(val, buf, u);
 				}
 				if ((attrib = tag.getAttribute("gloss"))) {
+					// I'm sure this is not the cleanest way to do it, but it gets the job done
+					// for rendering ruby chars properly ^_^
+					buf -= lastText.length();
+					
+					outText("<ruby><rb>", buf, u);
+					outText(lastText, buf, u);
 					val = strchr(attrib, ':');
 					val = (val) ? (val + 1) : attrib;
-					outText("(", buf, u);
+					outText("</rb><rp>(</rp><rt>", buf, u);
 					outText(val, buf, u);
-					outText(")", buf, u);
+					outText("</rt><rp>)</rp></ruby>", buf, u);
 				}
 				if (!morphFirst) {
 					processLemma(u->suspendTextPassThru, tag, buf);
@@ -272,8 +278,8 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 			}
 		}
 
-		// <p> paragraph tag
-		else if (!strcmp(tag.getName(), "p")) {
+		// <p> paragraph and <lg> linegroup tags
+		else if (!strcmp(tag.getName(), "p") || !strcmp(tag.getName(), "lg")) {
 			if ((!tag.isEndTag()) && (!tag.isEmpty())) {	// non-empty start tag
 				outText("<!P><br />", buf, u);
 			}
@@ -474,7 +480,7 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 		else if (!strcmp(tag.getName(), "hi")) {
 			SWBuf type = tag.getAttribute("type");
 			if ((!tag.isEndTag()) && (!tag.isEmpty())) {
-				if (type == "b" || type == "x-b") {
+				if (type == "bold" || type == "b" || type == "x-b") {
 					outText("<b>", buf, u);
 					u->inBold = true;
 				}
@@ -602,9 +608,9 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 			outText(URL::encode(u->version.c_str()).c_str(), buf, u);
 			outText("\">", buf, u);
 
-			outText("<img border=\"0\" src=\"", buf, u);
+			outText("<img src=\"file:", buf, u);
 			outText(filepath, buf, u);
-			outText("\" />", buf, u);
+			outText("\" border=\"0\" />", buf, u);
 
 			outText("</a>", buf, u);
 		}
