@@ -1,7 +1,7 @@
 #!/bin/sh
 #******************************************************************************
 #
-# $Id: verseparsing.sh 2329 2009-04-23 06:33:28Z benpmorgan $
+# $Id: verseparsing.sh 2476 2009-11-18 07:43:01Z scribe $
 #
 # Copyright 1998-2009 CrossWire Bible Society (http://www.crosswire.org)
 #	CrossWire Bible Society
@@ -80,6 +80,8 @@
 ../parsekey "1Jn 1:1; 3:10"
 ../parsekey "1Jn 1:1 ;3:10"
 ../parsekey "1Jn 1:1 ; 3:10"
+../parsekey "1Jn1.1f ; 3:10ff"
+../parsekey "1Jn1.1ff ; 3:10f"
 
 en_abbrevs="
 Gen Genes Exod Ex Exo Lev Le Levi Num Nu Numb Deut Deu De Josh Jos Joshu
