@@ -1,8 +1,26 @@
+/*
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
+
 #include <ctype.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <sys/stat.h>
 
 #ifndef __GNUC__
 #include <io.h>
@@ -10,17 +28,15 @@
 #include <unistd.h>
 #endif
 
+#include <filemgr.h>
 #include <swmgr.h>
 #include <rawtext.h>
 #include <iostream>
 #include <string>
 #include <versekey.h>
 
-#ifndef O_BINARY
-#define O_BINARY 0
-#endif
-
 #ifndef NO_SWORD_NAMESPACE
+using sword::FileMgr;
 using sword::SWMgr;
 using sword::RawText;
 using sword::VerseKey;
@@ -144,6 +160,7 @@ int main(int argc, char **argv) {
 	if (argc < 2) {
 //		fprintf(stderr, "usage: %s <vpl_file> </path/to/mod> [0|1 - file includes prepended verse references]\n", argv[0]);
 		fprintf(stderr, "usage: %s <source_vpl_file> </path/to/output/mod/> [0|1 - prepended verse refs] [0|1 - NT only]\n\n", argv[0]);
+		fprintf(stderr, "\tWARNING: THIS IS CURRENTLY A KJV-VERSIFICATION-ONLY UTILITY\n");
 		fprintf(stderr, "\tWith no verse refs, source file must contain exactly 31102 lines.\n");
 		fprintf(stderr, "\tThis is KJV verse count plus headings for MODULE,\n");
 		fprintf(stderr, "\tTESTAMENT, BOOK, CHAPTER. An example snippet follows:\n\n");
@@ -153,7 +170,7 @@ int main(int argc, char **argv) {
 		fprintf(stderr, "\t\tCHAPTER 1 HEADER\n");
 		fprintf(stderr, "\t\tIn the beginning...\n\n");
 		fprintf(stderr, "\t... implying there must also be a CHAPTER2 HEADER,\n");
-        fprintf(stderr, "\tEXODUS HEADER, NEW TESTAMENT HEADER, etc.  If there is no text for\n");
+		fprintf(stderr, "\tEXODUS HEADER, NEW TESTAMENT HEADER, etc.  If there is no text for\n");
 		fprintf(stderr, "\tthe header, a blank line must, at least, hold place.\n\n");
 		fprintf(stderr, "\tWith verse refs, source file must simply contain any number of lines,\n");
 		fprintf(stderr, "\tthat begin with the verse reference for which it is an entry.  e.g.:\n\n");
@@ -163,7 +180,7 @@ int main(int argc, char **argv) {
 	}
 
 	// Let's see if we can open our input file
-	int fd = open(argv[1], O_RDONLY|O_BINARY);
+	int fd = FileMgr::openFileReadOnly(argv[1]);
 	if (fd < 0) {
 		fprintf(stderr, "error: %s: couldn't open input file: %s \n", argv[0], argv[1]);
 		exit(-2);
